/* 
 * (c) 2015 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _EFFECTS_H_
#define _EFFECTS_H_

#include "config.h"
#include <gtk/gtk.h>
#include <gegl.h>


typedef enum{
	GEGL_DIBUJA_INSTA_CURVE_PRESET_NONE = 0,
	GEGL_DIBUJA_INSTA_CURVE_PRESET_1977,
	GEGL_DIBUJA_INSTA_CURVE_PRESET_BRANNAN,
	GEGL_DIBUJA_INSTA_CURVE_PRESET_GOTHAM,
	GEGL_DIBUJA_INSTA_CURVE_PRESET_NASHVILLE
}DibujaInstaCurve;


GeglBuffer *gegl_buffer_from_surface (cairo_surface_t *ARGB32_surface);
GeglBuffer *fx_copy_buffer(GeglBuffer *src);
cairo_surface_t *buffer_to_surface(GeglBuffer *buffer);

void fx_apply_operation (GeglBuffer *src_buffer, GeglNode *operation,
					GeglBuffer *dest_buffer, const GeglRectangle *dest_rect);

void fx_apply_grey (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
			   const GeglRectangle *dest_rect);

void fx_apply_gaussian_blur (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
						const GeglRectangle *dest_rect,
						gdouble std_dev_x, gdouble std_dev_y);


void fx_apply_invert (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
					  const GeglRectangle *dest_rect);

void fx_apply_brightness_contrast (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
								   const GeglRectangle *dest_rect, 
								   gdouble contrast, gdouble brightness);
void
fx_apply_pixelize (GeglBuffer *src_buffer,
                 GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect);

void fx_apply_insta_1977 (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect);

void fx_apply_insta_brannan (GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
                 const GeglRectangle *dest_rect);

void fx_apply_emboss(GeglBuffer *src_buffer, GeglBuffer *dest_buffer,
					 const GeglRectangle *dest_rect);

const Babl *fx_get_format_ARGB32(void);

/* In main.c */
void fx_update_fx_menu(void);

#endif

