/* 
 * (c) 2014 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "rotate.h"
#include "dibuja.h"
#include "util.h"
#include "selection.h"


static GtkWidget *dialog = NULL;
static GtkWidget *darea = NULL;
static double angle = 0.;

void dialog_rotate_button_close_activate(GtkWidget *widget, gpointer data)
{
	
}

void dialog_rotate_button_apply_activate(GtkWidget *widget, gpointer data)
{
	
}

void dialog_rotate_angle_spin_value_changed(GtkSpinButton *spin, gpointer data)
{
	angle = gtk_spin_button_get_value(spin);
	if(GTK_IS_WIDGET(darea) && gtk_widget_get_visible(darea)){
		gtk_widget_queue_draw(darea);
	}
}

void dialog_rotate_draw_rotated_image(GtkWidget *widget, cairo_t *cr, cairo_surface_t *surf)
{
	cairo_t *ct;
	cairo_surface_t *tmp;
	double w, h;
	double scale = 1;
	GtkAllocation allocation;

	gtk_widget_get_allocation(widget, &allocation);

	w = (double)cairo_image_surface_get_width(surf);
	h = (double)cairo_image_surface_get_height(surf);

	scale = MIN(allocation.width, allocation.height) / MAX(w, h);

	tmp = cairo_image_surface_create(DEF_SURF_FORMAT, (int)w * scale, (int)h * scale);
	ct = cairo_create(tmp);
	cairo_scale(ct, scale, scale);
	cairo_set_source_surface(ct, surf, 0., 0.);
	cairo_paint(ct);
	cairo_destroy(ct);

	w = (double)cairo_image_surface_get_width(tmp);
	h = (double)cairo_image_surface_get_height(tmp);

	cairo_translate(cr, (double)allocation.width / 2.,
						(double)allocation.height / 2.);
	cairo_rotate(cr, angle * M_PI / 180.);
	cairo_translate(cr, -w / 2.0, -h / 2.0);

	cairo_set_source_surface(cr, tmp, 0, 0);
	cairo_paint(cr);

	cairo_surface_destroy(tmp);
}
#if GTK_CHECK_VERSION(3, 0, 0)
gboolean dialog_rotate_darea_expose_event_cb(GtkWidget *widget,
										cairo_t *cr, gpointer data)
{
	cairo_surface_t *surf;

	if(sel_have())
		surf = sel_get_cur_image();
	else
		surf = dibuja_get_surface();

	dialog_rotate_draw_rotated_image(widget, cr, surf);

	return TRUE;
}
#else
gboolean dialog_rotate_darea_expose_event_cb(GtkWidget *widget,
										GdkEventExpose *event, gpointer data)
{
	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
	cairo_surface_t *surf;

	if(sel_have())
		surf = sel_get_cur_image();
	else
		surf = dibuja_get_surface();

	dialog_rotate_draw_rotated_image(widget, cr, surf);

	cairo_destroy(cr);

	return TRUE;
}
#endif

void dialog_rotate_run(void)
{
	gint ret = 0;

	if(!GTK_IS_WIDGET(dialog)){
		dialog = get_widget("dialog_rotate");
	}
	if(!GTK_IS_WIDGET(darea)){
		darea = get_widget("dialog_rotate_darea");
	}

	angle = 0.;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(get_widget("dialog_rotate_angle_spin")), angle);
	ret = gtk_dialog_run (GTK_DIALOG (dialog));
	switch (ret){
    	case GTK_RESPONSE_APPLY:
    		if(sel_have()){
    			sel_rotate(angle);
    		}
    		else{
    			dibuja_rotate_image(angle);
    		}
    		break;
    	default:
    		break;
	}

	gtk_widget_hide (dialog);
}
