/* 
 * (c) 2015 - 2023 Juan Balderas
 * This file is part of Dibuja.

 * Dibuja is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Dibuja is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Dibuja.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _UTIL_H_
	#define _UTIL_H_

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include "config.h"

#define COPY_YEAR	"2014 - 2023"
#define SEP         G_DIR_SEPARATOR_S

#define builder_get_widget(bldr,nm)  (GTK_WIDGET(gtk_builder_get_object((bldr),(nm))))

#if 0
#define WORKING_DIR		"/your/working/dir/here/"
#define UI_FILE_DIR		WORKING_DIR
  #if GTK_CHECK_VERSION(3, 0, 0)
    #define UI_FILE			WORKING_DIR "data/dibuja3.ui"
  #else
    #define UI_FILE			WORKING_DIR "data/dibuja.ui"
  #endif
#define DEF_BRUSH_DIR	WORKING_DIR "data/brushes/"
#define ICON_DIR_22x22  WORKING_DIR "data" SEP
#define DIBUJA_HELP_URI "file://" WORKING_DIR "doc" SEP "html" SEP "dibuja.htm"
#define DIBUJA_CURSOR_DIR	ICON_DIR_22x22
#define DIBUJA_README	PACKAGE_DATA_DIR SEP "doc" SEP PACKAGE_NAME SEP "README"

#else
#define UI_FILE_DIR		PACKAGE_DATA_DIR SEP PACKAGE_NAME SEP "ui" SEP
  #if GTK_CHECK_VERSION(3, 0, 0)
    #define UI_FILE				UI_FILE_DIR PACKAGE_NAME "3.ui"
  #else
    #define UI_FILE         UI_FILE_DIR PACKAGE_NAME ".ui"
  #endif
#define DEF_BRUSH_DIR	PACKAGE_DATA_DIR SEP PACKAGE_NAME SEP "brushes" SEP

#define ICON_DIR_22x22  UI_FILE_DIR
#define DIBUJA_CURSOR_DIR	ICON_DIR_22x22

#define DIBUJA_HELP_URI "file://" PACKAGE_DATA_DIR  SEP "doc" SEP "dibuja" SEP "dibuja.htm"
#define DIBUJA_README	PACKAGE_DATA_DIR SEP "doc" SEP PACKAGE_NAME SEP "README"

#endif

#define DIBUJA_CONFIG_FILE	"dibuja.conf"

#define DIBUJA_CURSOR	((GDK_LAST_CURSOR)+(1))
#define ICON_DIR_48x48  PACKAGE_DATA_DIR SEP "icons"
#define DIBUJA_LOGO		PACKAGE_DATA_DIR SEP "pixmaps" SEP PACKAGE_NAME ".png"


#define CANVAS_DEF_WIDTH	(600)
#define CANVAS_DEF_HEIGHT	(400)

/*  dots per inch. From the gimp
 */
#define GIMP_MIN_RESOLUTION  (5e-3)      /*  shouldn't display as 0.000  */
#define GIMP_MAX_RESOLUTION  (65536.0)
#ifndef ROUND
	#define ROUND(x) ((int) ((x) + 0.5))
#endif
/*                            */


#define ZOOM_COMBO_100_PERC	(6)/* Default zoom combo index */
#define MIN_ZOOM_INT		(10)
#define MAX_ZOOM_INT		(1600)

#define DEF_DLG_WIDTH	(400)
#define DEF_DLG_HEIGHT	(300)
#define util_dialog_def_size(d)	gtk_window_set_default_size(GTK_WINDOW(d),   \
															DEF_DLG_WIDTH,   \
															DEF_DLG_HEIGHT)

#define STOCK_APPLY				"gtk-apply"
#define STOCK_CANCEL			"gtk-cancel"
#define STOCK_CLOSE				"gtk-close"
#define STOCK_NO				"gtk-no"
#define STOCK_OPEN				"gtk-open"
#define STOCK_OK				"gtk-ok"
#define STOCK_REFRESH			"gtk-refresh"
#define STOCK_REVERT_TO_SAVED	"gtk-revert-to-saved"
#define STOCK_SAVE				"gtk-save"
#define STOCK_YES				"gtk-yes"


gboolean do_quit(void);
gchar *text_view_get_text(GtkTextView *textview, gboolean hidden_chars);
int message_box_yes_no(gchar const *format, ...);
void message_box_error(gchar const *format, ...);
gint message_box_yes_no_cancel(gchar const *format, ...);
void message_box_ok(GtkMessageType type, gchar const *format, ...);

GtkWidget *get_widget(const gchar *name);
GtkWindow *get_window(void);
void util_save_widgets(GtkWidget *window, GtkBuilder *builder);
void set_update_rect(GdkRectangle *r, int p1x, int p1y, int p2x, int p2y, double z);

void util_get_display_info(GtkWindow *window, gint *w, gint *h,
						   gint *wmm, gint *hmm);
void gimp_get_screen_resolution (GtkWindow *window, gdouble *xres, gdouble *yres);

void util_set_tool_page(int tt, int page_num);

void canvas_set_cursor(GtkWidget *widget, GdkCursorType type);
void canvas_set_wait_cursor(GtkWidget *widget);

void util_spin_set_val(char *adj, gdouble value);
char *util_get_config_filename(void);
char *util_get_dibuja_config_dir(void);

GtkWidget *util_create_label(const gchar *format, ...);

void dbj_window_get_size(GdkWindow *window, gint *width, gint *height);

/* Combo box =================================================================*/
GtkWidget *dib_combo_box_entry_new_text(void);
void dib_combo_box_append_text(GtkWidget *combo, gchar *text);
char *dib_combo_box_get_active_text(GtkWidget *combo);
void dib_combo_box_set_active(GtkWidget *combo, gint index_);
gint dib_combo_box_get_active(GtkWidget *combo);
void dib_combo_box_insert_text(GtkWidget *combo, gint position,
							   const gchar *text);

GtkBuilder *create_builder(const char *filename, GError **error);

typedef enum
{
  DIB_EXPAND = 1 << 0,
  DIN_SHRINK = 1 << 1,
  DIB_FILL   = 1 << 2
} DibAttachOptions;

GtkWidget *dib_table_new(guint rows, guint columns, gboolean homogeneous);
void dib_table_attach(GtkWidget *table, GtkWidget *child,
					  guint left_attach, guint right_attach,
					  guint top_attach, guint bottom_attach,
					  DibAttachOptions xoptions, DibAttachOptions yoptions,
					  guint xpadding, guint ypadding);
void dib_table_attach_defaults(GtkWidget *table, GtkWidget *widget, 
							   guint lattach, guint rattach,
							   guint tattach, guint battach);
void dbj_dialog_add_button(GtkDialog *dlg, const char *name, const char *text,
						   gint id, gboolean def_resp);
GtkWidget *image_from_default_theme(const char *name, GtkIconSize size);
  #if GTK_CHECK_VERSION(3, 12, 0)
    void util_set_select_dlg_button_image(GtkDialog *dialog,
										int resp_id, const char *name);
  #endif
#endif
